/* 
;
Bildet ein Integral ber den gemessenen Strom.
Zeigt das Ergebnis in mAh an.
Ausserdem die Zeit und den aktuellen Strom.
Anzeige ber RS232 mglich

; Hardware
;              ___________
;       __    /           |      
;+5V O-|__|--|RESET    PC5|--O 
;            |            |      
;         O--|PD0      PC4|--O
;            |            |      
;         O--|PD1      PC3|--O
;            |            |     
;     LCD O--|PD2      PC2|--O
;            |            |
;     LCD O--|PD3     ADC1|--O 
;            |    A T     |
;     LCD O--|PD4     ADC0|--O Strom Eingang
;            |   mega     |
;     +5V O--|VCC      GND|--O GND
;            |     8      |    
;     GND O--|GND     AREF|--O
;            |            |      
;   XTAL1 O--|XTAL1   AVCC|--O
;            |            |
;   XTAL2 O--|XTAL2 PB5SCK|--O 
;            |            |
;         O--|PD5  PB4MISO|--O 
;            |            |
;         O--|PD6  PB3MOSI|--O LCD
;            |            |
;         O--|PD7  PB2OC1B|--O LCD
;            |            |
;     LCD O--|PB0  PB1OC1A|--O LCD
;            |____________|
;
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdint.h>
#include <Stdio.h>
#include "BinaryNr.h"

#define F_CPU            9600000     /* Quart 6,6 Mhz */
#define UART_BAUD_RATE      9600      /* 9600 baud */
#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)
#define Prescaler 1024;
#define startwert  65537 - F_CPU/Prescaler; //Zhler Startwert, berlauf bei 65536

#include "uart_routinen.c"
#include "lcd.h"
#include "lcd.c"
#include "sonst_routinen.c"

#define u8 unsigned char
#define u16 unsigned int

//Variablen
u8 y;
u8 sekundenH=0;
u8 minutenH=0;
u8 stunden=0;
u8 sekundenL=0;
u8 minutenL=0;

char s[4];  //hilfsvariablen
char z[4];


u16 adWert; // AD-Wert 
u16 strom;  // Strom 
u16 summe;  // Strom aussummieren
u16 mah;    // Umgerechnet 



/* Unterprogramme */
/* INIT */

void port_init(void){		// initialize ports 
  DDRC  = 0x20;             // PORTA als Eingang BIT5 (LED) Ausgang
}

void timer_init(void){		// timer 0 overflow 
  TCCR1B = B00000101;       // Prescaler auf 1024 stellen
  TIMSK  |= (1 << TOIE1);   // Timer 1 Overflow Interrupt enable
  TCNT1 = startwert;        // vorbesetzen (zhlt bis FFFF)
}

void ad_init(void){		    // initialize ad-wandler 
  ADMUX = 0x00;             // Kanal0
  ADMUX |= (1<<REFS1) | (1<<REFS0); //interne Referenzspannung 2,5V nutzen 
  ADCSR=B11000111;          // enable ADC
}

//lese ad0 (Strom)
int leseAD(void)
{ 
  uint16_t wert;
  uint8_t  highwert;
  ADCSR |= (1<<ADSC);              // eine Wandlung starten
  while ( ADCSR & (1<<ADSC) ) {;}  // auf Abschluss der Konvertierung warten
  wert     = ADCL;                 // Low  auslesen	
  highwert = ADCH;                 // High auslesen
  wert = wert + highwert*256;//	
  return wert;
}


//Interrupt wenn Zeichen per uart empfangen
ISR(SIG_UART_RECV)      
{
    y = UDR;                      // lese byte aus UART
}

//Anzeige der Ladung auf LCD
void zeige_Ladung(u16 mah){
  itoa(mah,s,10);				  // wert in ACSII	
  lcd_puts("Ladung: ");
  lcd_puts(s);                    
  lcd_puts(" mAh ");
}

//Anzeige der Zeit auf LCD
void zeige_Zeit(void){			//hh,mm,ss berechnen
  sekundenL++;                  // Sekunden zhlen
  if(sekundenL==10){
    sekundenH++;sekundenL=0;}
  if(sekundenH==6){
	minutenL++;sekundenH=0;}
  if(minutenL==10){
    minutenH++;minutenL=0;}
  if(minutenH==6){
    stunden++;minutenH=0;}
								// auf LCD ausgeben	
  itoa(stunden,s,10);lcd_puts(s);lcd_puts(":");
  itoa(minutenH,s,10);lcd_puts(s);
  itoa(minutenL,s,10);lcd_puts(s);lcd_puts(":");
  itoa(sekundenH,s,10);lcd_puts(s);
  itoa(sekundenL,s,10);lcd_puts(s);
}

//Anzeige aktueller Strom auf LCD
void zeige_Strom(u16 strom){
  itoa(strom,s,10);				 
  lcd_puts(s);                    
  lcd_puts(" mA");  
}


//Timer Interrupt - genau jede Sekunde
SIGNAL (TIMER1_OVF_vect)          // Timer1 overflow 
{
  cli();
  TCNT1 = startwert;              // Vorbesetzen (1s)

  adWert=leseAD();                // lese Spannungswert    
  strom = adWert;			      // 1024 = 1024mA (1 zu 1)

  itoa(strom,s,10);				  // wert in ACSII	
  uart_puts(s);		              // auf uart ausgeben	
  uart_puts(" ");		          // auf uart ausgeben	

  summe = summe + strom;          // Strom aussummieren

  if (summe >= 3600){			  // in mAh umrechnen	
    mah++; 						  // wenn 1mAh voll +1	 
	summe = summe % 3600;         // Rest weiterzhlen
  }	                       


  lcd_clrscr();					  // 1.Zeile
  zeige_Ladung(mah);			  // Ladung anzeigen	

  lcd_gotoxy(0,1);				  // 2. Zeile
  zeige_Zeit();					  // Zeit anzeigen

  lcd_gotoxy(10,1);				  // 2. Zeile Stelle 10
  zeige_Strom(strom);             // aktuellen Strom anzeigen

  sei();                          // enable interrupts
}



/*********************/
int main()
{
  port_init(); 			// alles initialisieren
  uart_init();
  ad_init();
  timer_init();
  sei();                // enable interrupts	
  lcd_init(LCD_DISP_ON);// initialisiere display, cursor off 
  for(;;){}             // tut nix, Endlosschleife
}



